@@ -104,41 +104,91 @@ Another interesting feature is to convert ASTs to YAML:
 ]
 ```
 '''
-
-from typing import Any, Callable, List, Mapping, Optional, Sequence, Set, Tuple, Union, cast
-from functools import singledispatch
 from copy import copy
-from re import fullmatch
 from enum import Enum, auto
+from functools import lru_cache, partial, singledispatch
+from re import fullmatch
+from typing import (
+    Any,
+    Callable,
+    Container,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Union,
+    cast,
+)
 
 import clingo
 from clingo import ast
 from clingo.ast import (
-    AST, ASTSequence, ASTType, Function, Location, Position, StrSequence,
-    SymbolicTerm, SymbolicAtom, TheoryAtomType, TheoryFunction,
-    TheoryOperatorType, Transformer, UnaryOperation)
+    AST,
+    ASTSequence,
+    ASTType,
+    Function,
+    Location,
+    Position,
+    Sign,
+    StrSequence,
+    SymbolicAtom,
+    SymbolicTerm,
+    TheoryAtomType,
+    TheoryFunction,
+    TheoryOperatorType,
+    Transformer,
+    UnaryOperation,
+    parse_string,
+)
+
 from .theory import is_operator
 
-__all__ = ['ast_to_dict', 'dict_to_ast', 'location_to_str',
-           'prefix_symbolic_atoms', 'rename_symbolic_atoms', 'str_to_location',
-           'theory_parser_from_definition', 'Arity', 'Associativity',
-           'AtomTable', 'OperatorTable', 'TheoryParser', 'TheoryTermParser',
-           'TheoryUnparsedTermParser', 'TheoryUnparsedTermParser']
+__all__ = [
+    "Arity",
+    "Associativity",
+    "ASTPredicate",
+    "AtomTable",
+    "OperatorTable",
+    "TheoryParser",
+    "TheoryTermParser",
+    "TheoryUnparsedTermParser",
+    "ast_to_dict",
+    "clingo_literal_parser",
+    "clingo_term_parser",
+    "dict_to_ast",
+    "filter_body_literals",
+    "location_to_str",
+    "negate_sign",
+    "parse_theory",
+    "partition_body_literals",
+    "prefix_symbolic_atoms",
+    "reify_symbolic_atoms",
+    "rename_symbolic_atoms",
+    "str_to_location",
+    "theory_parser_from_definition",
+    "theory_term_to_literal",
+    "theory_term_to_term",
+]
 
 
 class Arity(Enum):
-    '''
+    """
     Enumeration of operator arities.
-    '''
+    """
+
     # pylint:disable=invalid-name
     Unary = 1
     Binary = 2
 
 
 class Associativity(Enum):
-    '''
+    """
     Enumeration of operator associativities.
-    '''
+    """
+
     # pylint: disable=invalid-name
     Left = auto()
     Right = auto()
@@ -146,18 +196,18 @@ class Associativity(Enum):
 
 
 def _s(m, a: str, b: str):
-    '''
+    """
     Select the match group b if not None and group a otherwise.
-    '''
+    """
     return m[a] if m[b] is None else m[b]
 
 
 def _quote(s: str) -> str:
-    return s.replace('\\', '\\\\').replace(':', '\\:')
+    return s.replace("\\", "\\\\").replace(":", "\\:")
 
 
 def _unquote(s: str) -> str:
-    return s.replace('\\:', ':').replace('\\\\', '\\')
+    return s.replace("\\:", ":").replace("\\\\", "\\")
 
 
 def location_to_str(loc: Location) -> str:
@@ -212,21 +262,23 @@ def str_to_location(loc: str) -> Location:
     location_to_str
     """
     m = fullmatch(
-        r'(?P<bf>([^\\:]|\\\\|\\:)*):(?P<bl>[0-9]*):(?P<bc>[0-9]+)'
-        r'(-(((?P<ef>([^\\:]|\\\\|\\:)*):)?(?P<el>[0-9]*):)?(?P<ec>[0-9]+))?', loc)
+        r"(?P<bf>([^\\:]|\\\\|\\:)*):(?P<bl>[0-9]*):(?P<bc>[0-9]+)"
+        r"(-(((?P<ef>([^\\:]|\\\\|\\:)*):)?(?P<el>[0-9]*):)?(?P<ec>[0-9]+))?",
+        loc,
+    )
     if not m:
-        raise RuntimeError('could not parse location')
-    begin = Position(_unquote(m['bf']), int(m['bl']), int(m['bc']))
-    end = Position(_unquote(_s(m, 'bf', 'ef')), int(_s(m, 'bl', 'el')), int(_s(m, 'bc', 'ec')))
+        raise RuntimeError("could not parse location")
+    begin = Position(_unquote(m["bf"]), int(m["bl"]), int(m["bc"]))
+    end = Position(
+        _unquote(_s(m, "bf", "ef")), int(_s(m, "bl", "el")), int(_s(m, "bc", "ec"))
+    )
     return Location(begin, end)
 
 
-OperatorTable = Mapping[Tuple[str, Arity],
-                        Tuple[int, Associativity]]
-AtomTable = Mapping[Tuple[str, int],
-                    Tuple[TheoryAtomType,
-                          str,
-                          Optional[Tuple[List[str], str]]]]
+OperatorTable = Mapping[Tuple[str, Arity], Tuple[int, Associativity]]
+AtomTable = Mapping[
+    Tuple[str, int], Tuple[TheoryAtomType, str, Optional[Tuple[List[str], str]]]
+]
 
 
 class TheoryUnparsedTermParser:
@@ -242,6 +294,7 @@ class TheoryUnparsedTermParser:
     table
         Mapping of operator/arity pairs to priority/associativity pairs.
     """
+
     _stack: List[Tuple[str, Arity]]
     _terms: List[AST]
     _table: OperatorTable
@@ -273,8 +326,9 @@ class TheoryUnparsedTermParser:
             return False
         priority, associativity = self._priority_and_associativity(operator)
         previous_priority = self._priority(*self._stack[-1])
-        return (previous_priority > priority or
-                (previous_priority == priority and associativity == Associativity.Left))
+        return previous_priority > priority or (
+            previous_priority == priority and associativity == Associativity.Left
+        )
 
     def _reduce(self) -> None:
         """
@@ -304,7 +358,9 @@ class TheoryUnparsedTermParser:
             Location of the operator for error reporting.
         """
         if (operator, arity) not in self._table:
-            raise RuntimeError(f"cannot parse operator `{operator}`: {location_to_str(location)}")
+            raise RuntimeError(
+                f"cannot parse operator `{operator}`: {location_to_str(location)}"
+            )
 
     def parse(self, x: AST) -> AST:
         """
@@ -360,10 +416,15 @@ class TheoryTermParser(Transformer):
     --------
     TheoryUnparsedTermParser
     """
+
     # pylint: disable=invalid-name
 
     def __init__(self, table: Union[OperatorTable, TheoryUnparsedTermParser]):
-        self._parser = table if isinstance(table, TheoryUnparsedTermParser) else TheoryUnparsedTermParser(table)
+        self._parser = (
+            table
+            if isinstance(table, TheoryUnparsedTermParser)
+            else TheoryUnparsedTermParser(table)
+        )
 
     def visit_TheoryFunction(self, x) -> AST:
         """
@@ -404,6 +465,52 @@ class TheoryTermParser(Transformer):
         return cast(AST, self(self._parser.parse(x)))
 
 
+_clingo_term_table = {
+    ("-", Arity.Unary): (5, Associativity.NoAssociativity),
+    ("~", Arity.Unary): (5, Associativity.NoAssociativity),
+    ("**", Arity.Binary): (4, Associativity.Right),
+    ("*", Arity.Binary): (3, Associativity.Left),
+    ("/", Arity.Binary): (3, Associativity.Left),
+    ("\\", Arity.Binary): (3, Associativity.Left),
+    ("+", Arity.Binary): (2, Associativity.Left),
+    ("-", Arity.Binary): (2, Associativity.Left),
+    ("&", Arity.Binary): (1, Associativity.Left),
+    ("?", Arity.Binary): (1, Associativity.Left),
+    ("^", Arity.Binary): (1, Associativity.Left),
+    ("..", Arity.Binary): (0, Associativity.Left),
+}
+
+
+@lru_cache(maxsize=None)
+def clingo_term_parser() -> TheoryTermParser:
+    """
+    Return a theory term parser that parses theory terms like clingo terms.
+
+    Note that for technical reasons pools and the absolute function are not
+    supported.
+    """
+    return TheoryTermParser(_clingo_term_table)
+
+
+@lru_cache(maxsize=None)
+def clingo_literal_parser() -> TheoryTermParser:
+    """
+    Return a theory term parser that parses theory literals similar to clingo's
+    parser for symbolic literals.
+
+    Note that for technical reasons pools and the absolute function are not
+    supported.
+    """
+    clingo_literal_table = _clingo_term_table.copy()
+    clingo_literal_table.update(
+        {
+            ("-", Arity.Unary): (0, Associativity.NoAssociativity),
+            ("not", Arity.Unary): (0, Associativity.NoAssociativity),
+        }
+    )
+    return TheoryTermParser(clingo_literal_table)
+
+
 class TheoryParser(Transformer):
     """
     This class parses theory atoms in the same way as clingo's internal parser.
@@ -418,16 +525,25 @@ class TheoryParser(Transformer):
         Mapping from atom name/arity pairs to tuples defining the acceptable
         structure of the theory atom.
     """
+
     # pylint: disable=invalid-name
-    _table: Mapping[Tuple[str, int],
-                    Tuple[TheoryAtomType,
-                          TheoryTermParser,
-                          Optional[Tuple[Set[str], TheoryTermParser]]]]
+    _table: Mapping[
+        Tuple[str, int],
+        Tuple[
+            TheoryAtomType,
+            TheoryTermParser,
+            Optional[Tuple[Set[str], TheoryTermParser]],
+        ],
+    ]
     _in_body: bool
     _in_head: bool
     _is_directive: bool
 
-    def __init__(self, terms: Mapping[str, Union[OperatorTable, TheoryTermParser]], atoms: AtomTable):
+    def __init__(
+        self,
+        terms: Mapping[str, Union[OperatorTable, TheoryTermParser]],
+        atoms: AtomTable,
+    ):
         self._reset()
 
         term_parsers = {}
@@ -578,15 +694,25 @@ class TheoryParser(Transformer):
         name = x.term.name
         arity = len(x.term.arguments)
         if (name, arity) not in self._table:
-            raise RuntimeError(f"theory atom definiton not found: {location_to_str(x.location)}")
+            raise RuntimeError(
+                f"theory atom definiton not found: {location_to_str(x.location)}"
+            )
 
         type_, element_parser, guard_table = self._table[(name, arity)]
         if type_ == TheoryAtomType.Head and not self._in_head:
-            raise RuntimeError(f"theory atom only accepted in head: {location_to_str(x.location)}")
+            raise RuntimeError(
+                f"theory atom only accepted in head: {location_to_str(x.location)}"
+            )
         if type_ == TheoryAtomType.Body and not self._in_body:
-            raise RuntimeError(f"theory atom only accepted in body: {location_to_str(x.location)}")
-        if type_ == TheoryAtomType.Directive and not (self._in_head and self._is_directive):
-            raise RuntimeError(f"theory atom must be a directive: {location_to_str(x.location)}")
+            raise RuntimeError(
+                f"theory atom only accepted in body: {location_to_str(x.location)}"
+            )
+        if type_ == TheoryAtomType.Directive and not (
+            self._in_head and self._is_directive
+        ):
+            raise RuntimeError(
+                f"theory atom must be a directive: {location_to_str(x.location)}"
+            )
 
         x = copy(x)
         x.term = element_parser(x.term)
@@ -594,11 +720,15 @@ class TheoryParser(Transformer):
 
         if x.guard is not None:
             if guard_table is None:
-                raise RuntimeError(f"unexpected guard in theory atom: {location_to_str(x.location)}")
+                raise RuntimeError(
+                    f"unexpected guard in theory atom: {location_to_str(x.location)}"
+                )
 
             guards, guard_parser = guard_table
             if x.guard.operator_name not in guards:
-                raise RuntimeError(f"unexpected guard in theory atom: {location_to_str(x.location)}")
+                raise RuntimeError(
+                    f"unexpected guard in theory atom: {location_to_str(x.location)}"
+                )
 
             x.guard = copy(x.guard)
             x.guard.term = guard_parser(x.guard.term)
@@ -648,22 +778,52 @@ def theory_parser_from_definition(x: AST) -> TheoryParser:
         if atom_def.guard is not None:
             guard = (atom_def.guard.operators, atom_def.guard.term)
 
-        atoms[(atom_def.name, atom_def.arity)] = (atom_def.atom_type, atom_def.term, guard)
+        atoms[(atom_def.name, atom_def.arity)] = (
+            atom_def.atom_type,
+            atom_def.term,
+            guard,
+        )
 
     return TheoryParser(terms, atoms)
 
 
+def parse_theory(s: str) -> TheoryParser:
+    """
+    Turn the given theory into a parser.
+    """
+    parser = None
+
+    def extract(stm):
+        nonlocal parser
+        if stm.ast_type == ASTType.TheoryDefinition:
+            if parser is not None:
+                raise ValueError("multiple theory definitions")
+            parser = theory_parser_from_definition(stm)
+        else:
+            assert (
+                stm.ast_type == ASTType.Program
+                and stm.name == "base"
+                and not stm.parameters
+            )
+
+    parse_string(f"{s}.", extract)
+    if parser is None:
+        raise ValueError("no theory definition found")
+    return cast(TheoryParser, parser)
+
+
 class _SymbolicAtomTransformer(Transformer):
-    '''
+    """
     Transforms symbolic atoms with the given function.
-    '''
+    """
+
     # pylint: disable=invalid-name
 
     def __init__(self, transformer_function: Callable[[AST], AST]):
         self._transformer_function = transformer_function
 
     def visit_SymbolicAtom(self, x: AST) -> AST:
-        '''
+        """
         Transform the given symbolic.
 
         Parameters
@@ -674,14 +834,14 @@ class _SymbolicAtomTransformer(Transformer):
         Returns
         -------
         The rewritten AST.
-        '''
+        """
         term = x.symbol
         new_term = self._transformer_function(term)
         return x if new_term is term else SymbolicAtom(new_term)
 
 
 def rewrite_symbolic_atoms(x: AST, rewrite_function: Callable[[AST], AST]) -> AST:
-    '''
+    """
     Rewrite all symbolic atoms in the given AST node with the given function.
 
     Parameters
@@ -695,12 +855,12 @@ def rewrite_symbolic_atoms(x: AST, rewrite_function: Callable[[AST], AST]) -> AS
     Returns
     -------
     The rewritten AST.
-    '''
+    """
     return cast(AST, _SymbolicAtomTransformer(rewrite_function)(x))
 
 
 def rename_symbolic_atoms(x: AST, rename_function: Callable[[str], str]) -> AST:
-    '''
+    """
     Rename all symbolic atoms in the given AST node with the given function.
 
     Parameters
@@ -713,23 +873,30 @@ def rename_symbolic_atoms(x: AST, rename_function: Callable[[str], str]) -> AST:
     Returns
     -------
     The rewritten AST.
-    '''
+    """
+
     def renamer(term: AST):
         if term.ast_type == ASTType.UnaryOperation:
-            return UnaryOperation(term.location, term.operator_type, renamer(term.argument))
+            return UnaryOperation(
+                term.location, term.operator_type, renamer(term.argument)
+            )
         if term.ast_type == ASTType.SymbolicTerm:
             sym = term.symbol
             new_name = rename_function(sym.name)
-            return SymbolicTerm(term.location, clingo.Function(new_name, sym.arguments, sym.positive))
+            return SymbolicTerm(
+                term.location, clingo.Function(new_name, sym.arguments, sym.positive)
+            )
         if term.ast_type == ASTType.Function:
-            return Function(term.location, rename_function(term.name), term.arguments, term.external)
+            return Function(
+                term.location, rename_function(term.name), term.arguments, term.external
+            )
         return term
 
     return rewrite_symbolic_atoms(x, renamer)
 
 
 def prefix_symbolic_atoms(x: AST, prefix: str) -> AST:
-    '''
+    """
     Prefix all symbolic atoms in the given AST with the given string.
 
     Parameters
@@ -746,14 +913,17 @@ def prefix_symbolic_atoms(x: AST, prefix: str) -> AST:
     See Also
     --------
     rename_symbolic_atoms
-    '''
+    """
     return rename_symbolic_atoms(x, lambda s: prefix + s)
 
 
-def reify_symbolic_atoms(x: AST, name: str,
-                         argument_extender: Callable[[AST], Sequence[AST]] = None,
-                         reify_strong_negation: bool = False) -> AST:
-    '''
+def reify_symbolic_atoms(
+    x: AST,
+    name: str,
+    argument_extender: Callable[[AST], Sequence[AST]] = None,
+    reify_strong_negation: bool = False,
+) -> AST:
+    """
     Reify all symbolic atoms in the given AST node with the given name and
     function.
 
@@ -769,8 +939,8 @@ def reify_symbolic_atoms(x: AST, name: str,
         the correct position.
     reify_strong_negation
         Boolean indicating how to encode strong negation. If false, `-p(X)` is
-        reified as `-name(p(X))`. If true, then -p(X) is reified as
-        name(-p(X)). In the latter case, this means that stable models
+        reified as `-name(p(X))`. If true, then `-p(X)` is reified as
+        `name(-p(X))`. In the latter case, this means that stable models
         containing both `name(p(a))` and `name(-p(a))` are possible. Clingo
         style consistency can be restored by adding the constraint
         `:- name(X), name(-X), X<-X.`
@@ -778,11 +948,13 @@ def reify_symbolic_atoms(x: AST, name: str,
     Returns
     -------
     The rewritten AST.
-    '''
+    """
 
     def reifier(term: AST):
         if term.ast_type == ASTType.UnaryOperation and not reify_strong_negation:
-            return UnaryOperation(term.location, term.operator_type, reifier(term.argument))
+            return UnaryOperation(
+                term.location, term.operator_type, reifier(term.argument)
+            )
         arguments = argument_extender(term) if argument_extender else [term]
         return Function(term.location, name, arguments, False)
 
@@ -850,9 +1022,9 @@ def ast_to_dict(x: AST) -> dict:
     --------
     dict_to_ast
     """
-    ret = {"ast_type": str(x.ast_type).replace('ASTType.', '')}
+    ret = {"ast_type": str(x.ast_type).replace("ASTType.", "")}
     for key, val in x.items():
-        if key == 'location':
+        if key == "location":
             assert isinstance(val, Location)
             enc = location_to_str(val)
         else:
@@ -919,4 +1091,303 @@ def dict_to_ast(x: dict) -> AST:
     --------
     ast_to_dict
     """
-    return getattr(ast, x['ast_type'])(**{key: _decode(value, key) for key, value in x.items() if key != "ast_type"})
+    return getattr(ast, x["ast_type"])(
+        **{key: _decode(value, key) for key, value in x.items() if key != "ast_type"}
+    )
+
+
+ASTPredicate = Union[Callable[[AST], bool], bool]
+
+
+def _eval_predicate(predicate: ASTPredicate, arg: AST) -> bool:
+    if callable(predicate):
+        return predicate(arg)
+    return predicate
+
+
+def _body_literal_predicate(
+    lit: AST,
+    symbolic_atom_predicate: ASTPredicate = True,
+    theory_atom_predicate: ASTPredicate = True,
+    aggregate_predicate: ASTPredicate = True,
+    conditional_literal_predicate: ASTPredicate = True,
+    signs: Container[Sign] = (Sign.NoSign, Sign.Negation, Sign.DoubleNegation),
+) -> bool:
+    if lit.ast_type == ASTType.Literal:
+        atom = lit.atom
+        if lit.sign not in signs:
+            return False
+        if atom.ast_type == ASTType.SymbolicAtom:
+            return _eval_predicate(symbolic_atom_predicate, atom.symbol)
+        if atom.ast_type in (ASTType.Aggregate, ASTType.BodyAggregate):
+            return _eval_predicate(aggregate_predicate, atom)
+        if atom.ast_type == ASTType.TheoryAtom:
+            return _eval_predicate(theory_atom_predicate, atom)
+    elif lit.ast_type == ASTType.ConditionalLiteral:
+        return lit.literal.sign in signs and _eval_predicate(
+            conditional_literal_predicate, lit
+        )
+    return True
+
+
+def filter_body_literals(
+    body: Iterable[AST],
+    symbolic_atom_predicate: ASTPredicate = True,
+    theory_atom_predicate: ASTPredicate = True,
+    aggregate_predicate: ASTPredicate = True,
+    conditional_literal_predicate: ASTPredicate = True,
+    signs: Container[Sign] = (Sign.NoSign, Sign.Negation, Sign.DoubleNegation),
+) -> Iterable[AST]:
+    """
+    Filters the given body literals according to the given predicates.
+
+    Parameters
+    ----------
+    body
+        An iterable of `AST`s for body literals.
+    symbolic_atom_predicate
+        Predicate to filter symbolic atoms.
+    theory_atom_predicate
+        Predicate to filter theory atoms.
+    aggregate_predicate
+        Predicate to filter aggregates.
+    conditional_literal_predicate
+        Predicate to filter conditional literals.
+    signs
+        Only include literals with the given signs.
+
+    Returns
+    -------
+    An iterarable of body literals.
+
+    Notes
+    -----
+    An `ASTPredicate` is a callable that takes an `AST` and returns a Boolean.
+    Booleans `True` and `False` are also accepted, meaning that the predicate
+    is always `True` or `False`, respectively.
+    """
+    pred = partial(
+        _body_literal_predicate,
+        symbolic_atom_predicate=symbolic_atom_predicate,
+        theory_atom_predicate=theory_atom_predicate,
+        aggregate_predicate=aggregate_predicate,
+        conditional_literal_predicate=conditional_literal_predicate,
+        signs=signs,
+    )
+    return filter(pred, body)
+
+
+def partition_body_literals(
+    body: Iterable[AST],
+    symbolic_atom_predicate: ASTPredicate = True,
+    theory_atom_predicate: ASTPredicate = True,
+    aggregate_predicate: ASTPredicate = True,
+    conditional_literal_predicate: ASTPredicate = True,
+    signs: Container[Sign] = (Sign.NoSign, Sign.Negation, Sign.DoubleNegation),
+) -> Tuple[List[AST], List[AST]]:
+    """
+    Partition the given body literals according to the given predicates.
+
+    Parameters
+    ----------
+    body
+        An iterable of `AST` that represents a body.
+    symbolic_atom_predicate
+        Predicate to partition symbolic atoms.
+    theory_atom_predicate
+        Predicate to partition theory atoms.
+    aggregate_predicate
+        Predicate to partition aggregates.
+    conditional_literal_predicate
+        Predicate to partition conditional literals.
+    signs
+        Only include literals with the given signs in the first list.
+
+    Returns
+    -------
+    A pair of lists of body literals. The first iterable yields the literals
+    that satisfy the predicate while the second one yields the ones that do
+    not.
+
+    Notes
+    -----
+    An `ASTPredicate` is a callable that takes an `AST` and returns a Boolean.
+    Booleans `True` and `False` are also accepted, meaning that the predicate
+    is always `True` or `False`, respectively.
+    """
+    pred = partial(
+        _body_literal_predicate,
+        symbolic_atom_predicate=symbolic_atom_predicate,
+        theory_atom_predicate=theory_atom_predicate,
+        aggregate_predicate=aggregate_predicate,
+        conditional_literal_predicate=conditional_literal_predicate,
+        signs=signs,
+    )
+    part_a: List[AST] = []
+    part_b: List[AST] = []
+    for lit in body:
+        if pred(lit):
+            part_a.append(lit)
+        else:
+            part_b.append(lit)
+    return part_a, part_b
+
+
+_unary_operator_map = {
+    "-": ast.UnaryOperator.Minus,
+    "~": ast.UnaryOperator.Negation,
+    "|": ast.UnaryOperator.Absolute,
+}
+
+_binary_operator_map = {
+    "+": ast.BinaryOperator.Plus,
+    "-": ast.BinaryOperator.Minus,
+    "*": ast.BinaryOperator.Multiplication,
+    "/": ast.BinaryOperator.Division,
+    "\\": ast.BinaryOperator.Modulo,
+    "**": ast.BinaryOperator.Power,
+    "&": ast.BinaryOperator.And,
+    "?": ast.BinaryOperator.Or,
+    "^": ast.BinaryOperator.XOr,
+}
+
+
+def _theory_term_to_term(x: AST) -> AST:
+    """
+    Convert a given theory term into a plain clingo term.
+    """
+    if x.ast_type in (ASTType.SymbolicTerm, ASTType.Variable):
+        return x
+
+    if x.ast_type == ASTType.TheoryFunction:
+        if len(x.arguments) == 1 and x.name in _unary_operator_map:
+            arg = _theory_term_to_term(x.arguments[0])
+            uop = _unary_operator_map[x.name]
+
+            return ast.UnaryOperation(x.location, uop, arg)
+
+        if len(x.arguments) == 2:
+            lhs = _theory_term_to_term(x.arguments[0])
+            rhs = _theory_term_to_term(x.arguments[1])
+
+            if x.name in _binary_operator_map:
+                bop = _binary_operator_map[x.name]
+                return ast.BinaryOperation(x.location, bop, lhs, rhs)
+
+            if x.name == "..":
+                return ast.Interval(x.location, lhs, rhs)
+
+        if not is_operator(x.name):
+            return ast.Function(
+                x.location,
+                x.name,
+                [_theory_term_to_term(a) for a in x.arguments],
+                False,
+            )
+
+    elif x.ast_type == ASTType.TheorySequence:
+        if x.sequence_type == ast.TheorySequenceType.Tuple:
+            return ast.Function(
+                x.location, "", [_theory_term_to_term(a) for a in x.terms], False
+            )
+
+    raise RuntimeError(f"{location_to_str(x.location)}: invalid term `{x}`")
+
+
+def theory_term_to_term(x: AST, parse: bool = True) -> AST:
+    """
+    Convert the given theory term into a plain clingo term.
+
+    If argument `parse` is set to true, occurences of unparsed theory terms are
+    parsed using `clingo_term_parser()`.
+    """
+    if parse:
+        x = clingo_term_parser()(x)
+    return _theory_term_to_term(x)
+
+
+def _build_atom(
+    location: ast.Location, positive: bool, name: str, arguments: List
+) -> ast.AST:
+    """
+    Helper function to create an atom.
+
+    Arguments:
+    location  -- Location to use.
+    positive  -- Classical sign of the atom.
+    name      -- The name of the atom.
+    arguments -- The arguments of the atom.
+    """
+    ret = ast.Function(location, name, arguments, False)
+    if not positive:
+        ret = ast.UnaryOperation(location, ast.UnaryOperator.Minus, ret)
+    return ast.SymbolicAtom(ret)
+
+
+def negate_sign(sign: ast.Sign) -> ast.Sign:
+    """
+    Negate the given sign.
+    """
+    if sign == ast.Sign.Negation:
+        return ast.Sign.DoubleNegation
+    return ast.Sign.Negation
+
+
+def _theory_term_to_literal(
+    x: AST, positive: bool = True, sign: ast.Sign = ast.Sign.NoSign
+) -> AST:
+    """
+    Convert a given theory term into a symbolic clingo literal.
+    """
+    if x.ast_type == ASTType.TheoryFunction:
+        if x.name == "-":
+            return _theory_term_to_literal(x.arguments[0], not positive, sign)
+
+        if x.name == "not":
+            sign = negate_sign(sign)
+            if not positive:
+                sign = negate_sign(sign)
+            return _theory_term_to_literal(x.arguments[0], True, sign)
+
+        if not is_operator(x.name):
+            atom = _build_atom(
+                x.location,
+                positive,
+                x.name,
+                [theory_term_to_term(a) for a in x.arguments],
+            )
+            return ast.Literal(x.location, sign, atom)
+
+    elif (
+        x.ast_type == ASTType.SymbolicTerm
+        and x.symbol.type == clingo.SymbolType.Function
+        and x.symbol.name
+    ):
+        atom = _build_atom(
+            x.location,
+            (positive == x.symbol.positive),
+            x.symbol.name,
+            [ast.SymbolicTerm(x.location, a) for a in x.symbol.arguments],
+        )
+        return ast.Literal(x.location, sign, atom)
+
+    raise RuntimeError(f"{location_to_str(x.location)}: invalid literal `{x}`")
+
+
+def theory_term_to_literal(x: AST, parse: bool = True) -> AST:
+    """
+    Convert the given theory term into a symbolic clingo literal.
+
+    If argument `parse` is set to true, occurences of unparsed theory terms are
+    parsed using `clingo_literal_parser()`.
+
+    Literals can use an arbitrary number of classical and default negation
+    signs. They are normalized using the following equivalences:
+
+    - `- - lit = lit`
+    - `- not lit = not not lit`
+    - `not not not lit = not lit`
+    """
+    if parse:
+        x = clingo_literal_parser()(x)
+    return _theory_term_to_literal(x, True, ast.Sign.NoSign)
