@@ -1,4 +1,4 @@
-'''
+"""
 This module provides functions to work with ground programs.
 
 This includes constructing a ground representation using an observer, pretty
@@ -38,20 +38,45 @@ c :- b.
 a
 b c a
 ```
-'''
+"""
 
-from typing import (Callable, Iterable, List, Mapping, MutableMapping, MutableSequence, NamedTuple, Optional, Sequence,
-                    Tuple, TypeVar)
+from copy import copy
 from dataclasses import dataclass, field
 from functools import singledispatch
 from itertools import chain
-from copy import copy
+from typing import (
+    Callable,
+    Iterable,
+    List,
+    Mapping,
+    MutableMapping,
+    MutableSequence,
+    NamedTuple,
+    Optional,
+    Sequence,
+    Tuple,
+    TypeVar,
+)
 
 from clingo import Backend, HeuristicType, Observer, Symbol, TruthValue
 
-__all__ = ['add_to_backend', 'pretty_str', 'remap', 'Edge', 'External', 'Fact',
-           'Heuristic', 'Minimize', 'Program', 'ProgramObserver', 'Project',
-           'Remapping', 'Rule', 'Show', 'WeightRule']
+__all__ = [
+    "add_to_backend",
+    "pretty_str",
+    "remap",
+    "Edge",
+    "External",
+    "Fact",
+    "Heuristic",
+    "Minimize",
+    "Program",
+    "ProgramObserver",
+    "Project",
+    "Remapping",
+    "Rule",
+    "Show",
+    "WeightRule",
+]
 __pdoc__ = {}
 
 Atom = int
@@ -59,13 +84,23 @@ Literal = int
 Weight = int
 OutputTable = Mapping[Atom, Symbol]
 AtomMap = Callable[[Atom], Atom]
-Statement = TypeVar('Statement', 'Fact', 'Show', 'Rule', 'WeightRule', 'Heuristic', 'Edge', 'Minimize', 'External',
-                    'Project')
+Statement = TypeVar(
+    "Statement",
+    "Fact",
+    "Show",
+    "Rule",
+    "WeightRule",
+    "Heuristic",
+    "Edge",
+    "Minimize",
+    "External",
+    "Project",
+)
 
 
 @singledispatch
-def pretty_str(stm, output_atoms: OutputTable) -> str:  # pylint: disable=unused-argument
-    '''
+def pretty_str(stm, output_atoms: OutputTable) -> str:
+    """
     Pretty print statements.
 
     Parameters
@@ -78,13 +113,14 @@ def pretty_str(stm, output_atoms: OutputTable) -> str:  # pylint: disable=unused
     Returns
     -------
     The string representation of the statement.
-    '''
-    assert False, 'unexpected type'
+    """
+    # pylint: disable=unused-argument
+    assert False, "unexpected type"
 
 
 @singledispatch
-def remap(stm, mapping: AtomMap):  # pylint: disable=unused-argument
-    '''
+def remap(stm, mapping: AtomMap):
+    """
     Remap literals in the given statement with the provided mapping.
 
     Parameters
@@ -101,13 +137,14 @@ def remap(stm, mapping: AtomMap):  # pylint: disable=unused-argument
     See Also
     --------
     Remapping
-    '''
-    assert False, 'unexpected type'
+    """
+    # pylint: disable=unused-argument
+    assert False, "unexpected type"
 
 
 @singledispatch
-def add_to_backend(stm, backend: Backend):  # pylint: disable=unused-argument
-    '''
+def add_to_backend(stm, backend: Backend):
+    """
     Add statements to the backend using the provided mapping to map literals.
 
     Parameters
@@ -116,51 +153,54 @@ def add_to_backend(stm, backend: Backend):  # pylint: disable=unused-argument
         The statement to add to the backend.
     backend
         The backend.
-    '''
-    assert False, 'unexpected type'
+    """
+    # pylint: disable=unused-argument
+    assert False, "unexpected type"
 
 
 def _pretty_str_lit(stm: Literal, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print literals and atoms.
-    '''
+    """
     atom = abs(stm)
     if atom in output_atoms:
         atom_str = str(output_atoms[atom])
     else:
-        atom_str = f'__x{atom}'
+        atom_str = f"__x{atom}"
 
-    return f'not {atom_str}' if stm < 0 else atom_str
+    return f"not {atom_str}" if stm < 0 else atom_str
 
 
-def _pretty_str_rule_head(choice: bool, has_body: bool, head: Sequence[Atom], output_atoms: OutputTable) -> str:
-    '''
+def _pretty_str_rule_head(
+    choice: bool, has_body: bool, head: Sequence[Atom], output_atoms: OutputTable
+) -> str:
+    """
     Pretty print the head of a rule including the implication symbol if
     necessary.
-    '''
-    ret = ''
+    """
+    ret = ""
 
     if choice:
-        ret += '{'
-    ret += '; '.join(_pretty_str_lit(lit, output_atoms) for lit in head)
+        ret += "{"
+    ret += "; ".join(_pretty_str_lit(lit, output_atoms) for lit in head)
     if choice:
-        ret += '}'
+        ret += "}"
 
     if has_body or (not head and not choice):
-        ret += ' :- '
+        ret += " :- "
 
     return ret
 
 
 def _pretty_str_truth_value(stm: TruthValue):
-    '''
+    """
     Pretty print a truth value.
-    '''
+    """
     if stm == TruthValue.False_:
-        return 'False'
+        return "False"
     if stm == TruthValue.True_:
-        return 'True'
-    return 'Free'
+        return "True"
+    return "Free"
 
 
 def _remap_lit(literal: Literal, mapping: AtomMap) -> Atom:
@@ -168,31 +208,33 @@ def _remap_lit(literal: Literal, mapping: AtomMap) -> Atom:
 
 
 def _remap_seq(literals: Sequence[Literal], mapping: AtomMap):
-    '''
+    """
     Apply the mapping to a sequence of literals or atoms.
-    '''
+    """
     return [_remap_lit(lit, mapping) for lit in literals]
 
 
 def _remap_wseq(literals: Sequence[Tuple[Literal, Weight]], mapping: AtomMap):
-    '''
+    """
     Apply the mapping to a sequence of weighted literals or atoms.
-    '''
+    """
     return [(_remap_lit(lit, mapping), weight) for lit, weight in literals]
 
 
 def _remap_stms(stms: MutableSequence[Statement], mapping: AtomMap):
-    '''
+    """
     Remap the given statements.
-    '''
+    """
     for i, stm in enumerate(stms):
         stms[i] = remap(stm, mapping)
 
 
-def _add_stms_to_backend(stms: Iterable[Statement], backend: Backend, mapping: Optional[AtomMap]):
-    '''
+def _add_stms_to_backend(
+    stms: Iterable[Statement], backend: Backend, mapping: Optional[AtomMap]
+):
+    """
     Remap the given statements returning a list with the result.
-    '''
+    """
     for stm in stms:
         if mapping:
             add_to_backend(remap(stm, mapping), backend)
@@ -201,77 +243,84 @@ def _add_stms_to_backend(stms: Iterable[Statement], backend: Backend, mapping: O
 
 
 class Fact(NamedTuple):
-    '''
+    """
     Ground representation of a fact.
-    '''
+    """
+
     symbol: Symbol
 
 
 @pretty_str.register(Fact)
-def _pretty_str_fact(stm: Fact, output_atoms: OutputTable) -> str:  # pylint: disable=unused-argument
-    '''
+def _pretty_str_fact(stm: Fact, output_atoms: OutputTable) -> str:
+    """
     Pretty print a fact.
-    '''
-    return f'{stm.symbol}.'
+    """
+    # pylint: disable=unused-argument
+    return f"{stm.symbol}."
 
 
 @remap.register(Fact)
-def _remap_fact(stm: Fact, mapping: AtomMap) -> Fact:  # pylint: disable=unused-argument
-    '''
+def _remap_fact(stm: Fact, mapping: AtomMap) -> Fact:
+    """
     Remap a fact statement.
-    '''
+    """
+    # pylint: disable=unused-argument
     return stm
 
 
 @add_to_backend.register(Fact)
-def _add_to_backend_fact(stm: Fact, backend: Backend) -> None:  # pylint: disable=unused-argument
-    '''
+def _add_to_backend_fact(stm: Fact, backend: Backend) -> None:
+    """
     Add a fact to the backend.
 
     This does nothing to not interfere with the mapping of literals. If facts
     are to be mapped, then this should be done manually beforehand.
-    '''
+    """
+    # pylint: disable=unused-argument
 
 
 class Show(NamedTuple):
-    '''
+    """
     Ground representation of a show statements.
-    '''
+    """
+
     symbol: Symbol
     condition: Sequence[Literal]
 
 
 @pretty_str.register(Show)
 def _pretty_str_show(stm: Show, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print a fact.
-    '''
-    body = ', '.join(_pretty_str_lit(lit, output_atoms) for lit in stm.condition)
+    """
+    body = ", ".join(_pretty_str_lit(lit, output_atoms) for lit in stm.condition)
     return f'#show {stm.symbol}{": " if body else ""}{body}.'
 
 
 @remap.register(Show)
 def _remap_show(stm: Show, mapping: AtomMap) -> Show:
-    '''
+    """
     Remap a show statetment.
-    '''
+    """
     return Show(stm.symbol, _remap_seq(stm.condition, mapping))
 
 
 @add_to_backend.register(Show)
-def _add_to_backend_show(stm: Show, backend: Backend) -> None:  # pylint: disable=unused-argument
-    '''
+def _add_to_backend_show(stm: Show, backend: Backend) -> None:
+    """
     Add a show statement to the backend.
 
     Note that this currently does nothing because backend does not yet support
     adding to the symbol table.
-    '''
+    """
+    # pylint: disable=unused-argument
 
 
 class Rule(NamedTuple):
-    '''
+    """
     Ground representation of disjunctive and choice rules.
-    '''
+    """
+
     choice: bool
     head: Sequence[Atom]
     body: Sequence[Literal]
@@ -279,35 +328,38 @@ class Rule(NamedTuple):
 
 @pretty_str.register(Rule)
 def _pretty_str_rule(stm: Rule, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print a rule.
-    '''
+    """
     head = _pretty_str_rule_head(stm.choice, bool(stm.body), stm.head, output_atoms)
-    body = ', '.join(_pretty_str_lit(lit, output_atoms) for lit in stm.body)
+    body = ", ".join(_pretty_str_lit(lit, output_atoms) for lit in stm.body)
 
-    return f'{head}{body}.'
+    return f"{head}{body}."
 
 
 @remap.register(Rule)
 def _remap_rule(stm: Rule, mapping: AtomMap) -> Rule:
-    '''
+    """
     Remap literals in a rule.
-    '''
-    return Rule(stm.choice, _remap_seq(stm.head, mapping), _remap_seq(stm.body, mapping))
+    """
+    return Rule(
+        stm.choice, _remap_seq(stm.head, mapping), _remap_seq(stm.body, mapping)
+    )
 
 
 @add_to_backend.register(Rule)
 def _add_to_backend_rule(stm: Rule, backend: Backend) -> None:
-    '''
+    """
     Add a rule to the backend.
-    '''
+    """
     backend.add_rule(stm.head, stm.body, stm.choice)
 
 
 class WeightRule(NamedTuple):
-    '''
+    """
     Ground representation of rules with a weight constraint in the body.
-    '''
+    """
+
     choice: bool
     head: Sequence[Atom]
     lower_bound: Weight
@@ -316,133 +368,146 @@ class WeightRule(NamedTuple):
 
 @pretty_str.register(WeightRule)
 def _pretty_str_weight_rule(stm: WeightRule, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print a rule or weight rule.
-    '''
+    """
     head = _pretty_str_rule_head(stm.choice, bool(stm.body), stm.head, output_atoms)
-    body = ', '.join(f'{weight},{i}: {_pretty_str_lit(literal, output_atoms)}'
-                     for i, (literal, weight) in enumerate(stm.body))
+    body = ", ".join(
+        f"{weight},{i}: {_pretty_str_lit(literal, output_atoms)}"
+        for i, (literal, weight) in enumerate(stm.body)
+    )
 
-    return f'{head}{stm.lower_bound}{{{body}}}.'
+    return f"{head}{stm.lower_bound}{{{body}}}."
 
 
 @remap.register(WeightRule)
 def _remap_weight_rule(stm: WeightRule, mapping: AtomMap) -> WeightRule:
-    '''
+    """
     Remap literals in a weight rule.
-    '''
-    return WeightRule(stm.choice, _remap_seq(stm.head, mapping), stm.lower_bound, _remap_wseq(stm.body, mapping))
+    """
+    return WeightRule(
+        stm.choice,
+        _remap_seq(stm.head, mapping),
+        stm.lower_bound,
+        _remap_wseq(stm.body, mapping),
+    )
 
 
 @add_to_backend.register(WeightRule)
 def _add_to_backend_weight_rule(stm: WeightRule, backend: Backend) -> None:
-    '''
+    """
     Add a weight rule to the backend.
-    '''
+    """
     backend.add_weight_rule(stm.head, stm.lower_bound, stm.body, stm.choice)
 
 
 class Project(NamedTuple):
-    '''
+    """
     Ground representation of project statements.
-    '''
+    """
+
     atom: Atom
 
 
 @pretty_str.register(Project)
 def _pretty_str_project(stm: Project, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print a project statement.
-    '''
-    return f'#project {_pretty_str_lit(stm.atom, output_atoms)}.'
+    """
+    return f"#project {_pretty_str_lit(stm.atom, output_atoms)}."
 
 
 @remap.register(Project)
 def _remap_project(stm: Project, mapping: AtomMap):
-    '''
+    """
     Remap project statement.
-    '''
+    """
     return Project(mapping(stm.atom))
 
 
 @add_to_backend.register(Project)
 def _add_to_backend_project(stm: Project, backend: Backend):
-    '''
+    """
     Add a project statement to the backend.
-    '''
+    """
     backend.add_project([stm.atom])
 
 
 class External(NamedTuple):
-    '''
+    """
     Ground representation of external atoms.
-    '''
+    """
+
     atom: Atom
     value: TruthValue
 
 
 @pretty_str.register(External)
 def _pretty_print_external(stm: External, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print an external.
-    '''
-    return f'#external {_pretty_str_lit(stm.atom, output_atoms)}. [{_pretty_str_truth_value(stm.value)}]'
+    """
+    return f"#external {_pretty_str_lit(stm.atom, output_atoms)}. [{_pretty_str_truth_value(stm.value)}]"
 
 
 @remap.register(External)
 def _remap_external(stm: External, mapping: AtomMap) -> External:
-    '''
+    """
     Remap the external.
-    '''
+    """
     return External(mapping(stm.atom), stm.value)
 
 
 @add_to_backend.register(External)
 def _add_to_backend_external(stm: External, backend: Backend):
-    '''
+    """
     Add an external statement to the backend remapping its atom.
-    '''
+    """
     backend.add_external(stm.atom, stm.value)
 
 
 class Minimize(NamedTuple):
-    '''
+    """
     Ground representation of a minimize statement.
-    '''
+    """
+
     priority: Weight
     literals: Sequence[Tuple[Literal, Weight]]
 
 
 @pretty_str.register(Minimize)
 def _pretty_print_minimize(stm, output_atoms) -> str:
-    '''
+    """
     Pretty print a minimize statement.
-    '''
-    body = '; '.join(f'{weight}@{stm.priority},{i}: {_pretty_str_lit(literal, output_atoms)}'
-                     for i, (literal, weight) in enumerate(stm.literals))
-    return f'#minimize{{{body}}}.'
+    """
+    body = "; ".join(
+        f"{weight}@{stm.priority},{i}: {_pretty_str_lit(literal, output_atoms)}"
+        for i, (literal, weight) in enumerate(stm.literals)
+    )
+    return f"#minimize{{{body}}}."
 
 
 @remap.register(Minimize)
 def _remap_minimize(stm: Minimize, mapping: AtomMap) -> Minimize:
-    '''
+    """
     Remap the literals in the minimize statement.
-    '''
+    """
     return Minimize(stm.priority, _remap_wseq(stm.literals, mapping))
 
 
 @add_to_backend.register(Minimize)
 def _add_to_backend_minimize(stm: Minimize, backend: Backend):
-    '''
+    """
     Add a minimize statement to the backend.
-    '''
+    """
     backend.add_minimize(stm.priority, stm.literals)
 
 
 class Heuristic(NamedTuple):
-    '''
+    """
     Ground representation of a heuristic statement.
-    '''
+    """
+
     atom: Atom
     type_: HeuristicType
     bias: Weight
@@ -451,15 +516,15 @@ class Heuristic(NamedTuple):
 
 
 def _pretty_str_heuristic_type(type_):
-    return str(type_).replace('HeuristicType.', '')
+    return str(type_).replace("HeuristicType.", "")
 
 
 @pretty_str.register(Heuristic)
 def _pretty_str_heuristic(stm: Heuristic, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print a heuristic statement.
-    '''
-    body = ', '.join(_pretty_str_lit(lit, output_atoms) for lit in stm.condition)
+    """
+    body = ", ".join(_pretty_str_lit(lit, output_atoms) for lit in stm.condition)
     head = _pretty_str_lit(stm.atom, output_atoms)
     type_ = _pretty_str_heuristic_type(stm.type_)
     return f'#heuristic {head}{": " if body else ""}{body}. [{stm.bias}@{stm.priority}, {type_}]'
@@ -467,24 +532,31 @@ def _pretty_str_heuristic(stm: Heuristic, output_atoms: OutputTable) -> str:
 
 @remap.register(Heuristic)
 def _remap_heuristic(stm: Heuristic, mapping: AtomMap) -> Heuristic:
-    '''
+    """
     Remap the heuristic statement.
-    '''
-    return Heuristic(mapping(stm.atom), stm.type_, stm.bias, stm.priority, _remap_seq(stm.condition, mapping))
+    """
+    return Heuristic(
+        mapping(stm.atom),
+        stm.type_,
+        stm.bias,
+        stm.priority,
+        _remap_seq(stm.condition, mapping),
+    )
 
 
 @add_to_backend.register(Heuristic)
 def _add_to_backend_heuristic(stm: Heuristic, backend: Backend) -> None:
-    '''
+    """
     Add a heurisitic statement to the backend.
-    '''
+    """
     backend.add_heuristic(stm.atom, stm.type_, stm.bias, stm.priority, stm.condition)
 
 
 class Edge(NamedTuple):
-    '''
+    """
     Ground representation of a heuristic statement.
-    '''
+    """
+
     u: int
     v: int
     condition: Sequence[Literal]
@@ -492,81 +564,82 @@ class Edge(NamedTuple):
 
 @pretty_str.register(Edge)
 def _pretty_str_edge(stm: Edge, output_atoms: OutputTable) -> str:
-    '''
+    """
     Pretty print a heuristic statement.
-    '''
-    body = ', '.join(_pretty_str_lit(lit, output_atoms) for lit in stm.condition)
+    """
+    body = ", ".join(_pretty_str_lit(lit, output_atoms) for lit in stm.condition)
     return f'#edge ({stm.u},{stm.v}){": " if body else ""}{body}.'
 
 
 @remap.register(Edge)
 def _remap_edge(stm: Edge, mapping: AtomMap) -> Edge:
-    '''
+    """
     Remap an edge statement.
-    '''
+    """
     return Edge(stm.u, stm.v, _remap_seq(stm.condition, mapping))
 
 
 @add_to_backend.register(Edge)
 def _add_to_backend_edge(stm: Edge, backend: Backend) -> None:
-    '''
+    """
     Add an edge statement to the backend remapping its literals.
-    '''
+    """
     backend.add_acyc_edge(stm.u, stm.v, stm.condition)
 
 
 @dataclass
 class Program:  # pylint: disable=too-many-instance-attributes
-    '''
+    """
     Ground program representation.
 
     Although inefficient, the string representation of this program is parsable
     by clingo.
-    '''
+    """
+
     output_atoms: MutableMapping[Atom, Symbol] = field(default_factory=dict)
-    '''
+    """
     A mapping from program atoms to symbols.
-    '''
+    """
     shows: List[Show] = field(default_factory=list)
-    '''
+    """
     A list of show statements.
-    '''
+    """
     facts: List[Fact] = field(default_factory=list)
-    '''
+    """
     A list of facts.
-    '''
+    """
     rules: List[Rule] = field(default_factory=list)
-    '''
+    """
     A list of rules.
-    '''
+    """
     weight_rules: List[WeightRule] = field(default_factory=list)
-    '''
+    """
     A list of weight rules.
-    '''
+    """
     heuristics: List[Heuristic] = field(default_factory=list)
-    '''
+    """
     A list of heuristic statements.
-    '''
+    """
     edges: List[Edge] = field(default_factory=list)
-    '''
+    """
     A list of edge statements.
-    '''
+    """
     minimizes: List[Minimize] = field(default_factory=list)
-    '''
+    """
     A list of minimize statements.
-    '''
+    """
     externals: List[External] = field(default_factory=list)
-    '''
+    """
     A list of external statements.
-    '''
+    """
     projects: Optional[List[Project]] = None
-    '''
+    """
     A list of project statements.
-    '''
+    """
     assumptions: List[Literal] = field(default_factory=list)
-    '''
+    """
     A list of assumptions in form of program literals.
-    '''
+    """
 
     def _pretty_stms(self, arg: Iterable[Statement], sort: bool) -> Iterable[str]:
         if sort:
@@ -586,18 +659,18 @@ class Program:  # pylint: disable=too-many-instance-attributes
         # This is to inform that there is an empty projection statement.
         # It might be worth to allow writing just #project.
         if not self.projects:
-            return ['#project x: #false.']
+            return ["#project x: #false."]
         arg = sorted(self.projects) if sort else self.projects
         return (pretty_str(project, self.output_atoms) for project in arg)
 
-    def sort(self) -> 'Program':
-        '''
+    def sort(self) -> "Program":
+        """
         Sort the statements in the program inplace.
 
         Returns
         -------
         A reference to self.
-        '''
+        """
         self.shows.sort()
         self.facts.sort()
         self.rules.sort()
@@ -612,8 +685,8 @@ class Program:  # pylint: disable=too-many-instance-attributes
 
         return self
 
-    def remap(self, mapping: AtomMap) -> 'Program':
-        '''
+    def remap(self, mapping: AtomMap) -> "Program":
+        """
         Remap the literals in the program inplace.
 
         Parameters
@@ -628,7 +701,7 @@ class Program:  # pylint: disable=too-many-instance-attributes
         See Also
         --------
         remap
-        '''
+        """
         _remap_stms(self.shows, mapping)
         _remap_stms(self.facts, mapping)
         _remap_stms(self.rules, mapping)
@@ -641,12 +714,16 @@ class Program:  # pylint: disable=too-many-instance-attributes
             _remap_stms(self.projects, mapping)
         for i, lit in enumerate(self.assumptions):
             self.assumptions[i] = _remap_lit(lit, mapping)
-        self.output_atoms = {mapping(lit): sym for lit, sym in self.output_atoms.items()}
+        self.output_atoms = {
+            mapping(lit): sym for lit, sym in self.output_atoms.items()
+        }
 
         return self
 
-    def add_to_backend(self, backend: Backend, mapping: Optional[AtomMap] = None) -> 'Program':
-        '''
+    def add_to_backend(
+        self, backend: Backend, mapping: Optional[AtomMap] = None
+    ) -> "Program":
+        """
         Add the program to the given backend with an optional mapping.
 
         Note that the output table cannot be added to the backend for technical
@@ -667,7 +744,7 @@ class Program:  # pylint: disable=too-many-instance-attributes
         See Also
         --------
         add_to_backend
-        '''
+        """
 
         _add_stms_to_backend(self.shows, backend, mapping)
         _add_stms_to_backend(self.facts, backend, mapping)
@@ -683,13 +760,14 @@ class Program:  # pylint: disable=too-many-instance-attributes
             else:
                 backend.add_project([])
 
-        backend.add_assume([_remap_lit(lit, mapping) if mapping else lit
-                            for lit in self.assumptions])
+        backend.add_assume(
+            [_remap_lit(lit, mapping) if mapping else lit for lit in self.assumptions]
+        )
 
         return self
 
     def pretty_str(self, sort: bool = True) -> str:
-        '''
+        """
         Return a readable string represenation of the program.
 
         Parameters
@@ -700,38 +778,41 @@ class Program:  # pylint: disable=too-many-instance-attributes
         Returns
         -------
         The string representation of the program.
-        '''
-        return '\n'.join(chain(
-            self._pretty_stms(self.shows, sort),
-            self._pretty_stms(self.facts, sort),
-            self._pretty_stms(self.rules, sort),
-            self._pretty_stms(self.weight_rules, sort),
-            self._pretty_stms(self.heuristics, sort),
-            self._pretty_stms(self.edges, sort),
-            self._pretty_stms(self.minimizes, sort),
-            self._pretty_stms(self.externals, sort),
-            self._pretty_projects(sort),
-            self._pretty_assumptions(sort)))
+        """
+        return "\n".join(
+            chain(
+                self._pretty_stms(self.shows, sort),
+                self._pretty_stms(self.facts, sort),
+                self._pretty_stms(self.rules, sort),
+                self._pretty_stms(self.weight_rules, sort),
+                self._pretty_stms(self.heuristics, sort),
+                self._pretty_stms(self.edges, sort),
+                self._pretty_stms(self.minimizes, sort),
+                self._pretty_stms(self.externals, sort),
+                self._pretty_projects(sort),
+                self._pretty_assumptions(sort),
+            )
+        )
 
-    def copy(self) -> 'Program':
-        '''
+    def copy(self) -> "Program":
+        """
         Return a shallow copy of the program copying all mutable state.
 
         Returns
         -------
         A shallow copy of the program.
-        '''
+        """
         return copy(self)
 
     def __str__(self) -> str:
-        '''
+        """
         Return a readable string represenation of the program.
-        '''
+        """
         return self.pretty_str()
 
 
 class Remapping:
-    '''
+    """
     This class maps existing literals to fresh literals as created by the
     backend.
 
@@ -743,11 +824,14 @@ class Remapping:
         The output table to initialize the mapping with.
     facts
         A list of facts each of which will receive a fresh program atom.
-    '''
+    """
+
     _backend: Backend
     _map: MutableMapping[Atom, Atom]
 
-    def __init__(self, backend: Backend, output_atoms: OutputTable, facts: Iterable[Fact] = None):
+    def __init__(
+        self, backend: Backend, output_atoms: OutputTable, facts: Iterable[Fact] = None
+    ):
         self._backend = backend
         self._map = {}
         for atom, sym in output_atoms.items():
@@ -758,7 +842,7 @@ class Remapping:
                 backend.add_rule([backend.add_atom(fact.symbol)])
 
     def __call__(self, atom: Atom) -> Atom:
-        '''
+        """
         Map the given program atom to the corresponding atom in the backend.
 
         If the literal was not mapped during initialization, a new literal is
@@ -772,18 +856,18 @@ class Remapping:
         Returns
         -------
         The remapped program atom.
-        '''
+        """
         if atom not in self._map:
             self._map[atom] = self._backend.add_atom()
 
         return self._map[atom]
 
 
-__pdoc__['Remapping.__call__'] = True
+__pdoc__["Remapping.__call__"] = True
 
 
 class ProgramObserver(Observer):
-    '''
+    """
     Program observer to build a ground program representation while grounding.
 
     This class explicitly ignores theory atoms because they already have a
@@ -793,35 +877,36 @@ class ProgramObserver(Observer):
     ----------
     program
         The program to add statements to.
-    '''
+    """
+
     _program: Program
 
     def __init__(self, program: Program):
         self._program = program
 
     def begin_step(self) -> None:
-        '''
+        """
         Resets the assumptions.
-        '''
+        """
         self._program.assumptions.clear()
 
     def output_atom(self, symbol: Symbol, atom: Atom) -> None:
-        '''
+        """
         Add the given atom to the list of facts or output table.
-        '''
+        """
         if atom != 0:
             self._program.output_atoms[atom] = symbol
         else:
             self._program.facts.append(Fact(symbol))
 
     def output_term(self, symbol: Symbol, condition: Sequence[Literal]) -> None:
-        '''
+        """
         Add a term to the output table.
-        '''
+        """
         self._program.shows.append(Show(symbol, condition))
 
     def rule(self, choice: bool, head: Sequence[Atom], body: Sequence[Literal]) -> None:
-        '''
+        """
         Add a rule to the ground representation.
 
         Parameters
@@ -832,12 +917,17 @@ class ProgramObserver(Observer):
             List of program atoms forming the rule head.
         body
             List of program literals forming the rule body.
-        '''
+        """
         self._program.rules.append(Rule(choice, head, body))
 
-    def weight_rule(self, choice: bool, head: Sequence[Atom], lower_bound: Weight,
-                    body: Sequence[Tuple[Literal, Weight]]) -> None:
-        '''
+    def weight_rule(
+        self,
+        choice: bool,
+        head: Sequence[Atom],
+        lower_bound: Weight,
+        body: Sequence[Tuple[Literal, Weight]],
+    ) -> None:
+        """
         Add a weight rule to the ground representation.
 
         Parameters
@@ -851,24 +941,24 @@ class ProgramObserver(Observer):
         body
             List of weighted literals (pairs of literal and weight) forming the
             elements of the weight constraint.
-        '''
+        """
         self._program.weight_rules.append(WeightRule(choice, head, lower_bound, body))
 
     def project(self, atoms: Sequence[Atom]) -> None:
-        '''
+        """
         Add a project statement to the ground representation.
 
         Parameters
         ----------
         atoms
             The program atoms to project on.
-        '''
+        """
         if self._program.projects is None:
             self._program.projects = []
         self._program.projects.extend(Project(atom) for atom in atoms)
 
     def external(self, atom: Atom, value: TruthValue) -> None:
-        '''
+        """
         Add an external statement to the ground representation.
 
         Parameters
@@ -877,11 +967,11 @@ class ProgramObserver(Observer):
             The external atom in form of a program literal.
         value
             The truth value of the external statement.
-        '''
+        """
         self._program.externals.append(External(atom, value))
 
     def assume(self, literals: Sequence[Literal]) -> None:
-        '''
+        """
         Extend the program with the given assumptions.
 
         Parameters
@@ -889,11 +979,13 @@ class ProgramObserver(Observer):
         literals
             The program literals to assume (positive literals are true and
             negative literals false for the next solve call).
-        '''
+        """
         self._program.assumptions.extend(literals)
 
-    def minimize(self, priority: Weight, literals: Sequence[Tuple[Literal, Weight]]) -> None:
-        '''
+    def minimize(
+        self, priority: Weight, literals: Sequence[Tuple[Literal, Weight]]
+    ) -> None:
+        """
         Add a minimize statement to the ground representation.
 
         Parameters
@@ -903,11 +995,11 @@ class ProgramObserver(Observer):
         literals
             List of weighted literals whose sum to minimize (pairs of literal
             and weight).
-        '''
+        """
         self._program.minimizes.append(Minimize(priority, literals))
 
     def acyc_edge(self, node_u: int, node_v: int, condition: Sequence[Literal]) -> None:
-        '''
+        """
         Add an edge statement to the gronud representation.
 
         Parameters
@@ -919,12 +1011,18 @@ class ProgramObserver(Observer):
         condition
             The list of program literals forming th condition under which to
             add the edge.
-        '''
+        """
         self._program.edges.append(Edge(node_u, node_v, condition))
 
-    def heuristic(self, atom: Atom, type_: HeuristicType, bias: Weight, priority: Weight,
-                  condition: Sequence[Literal]) -> None:
-        '''
+    def heuristic(
+        self,
+        atom: Atom,
+        type_: HeuristicType,
+        bias: Weight,
+        priority: Weight,
+        condition: Sequence[Literal],
+    ) -> None:
+        """
         Add heurisitic statement to the gronud representation.
 
         Parameters
@@ -939,5 +1037,7 @@ class ProgramObserver(Observer):
             An unsigned integer.
         condition
             List of program literals.
-        '''
-        self._program.heuristics.append(Heuristic(atom, type_, bias, priority, condition))
+        """
+        self._program.heuristics.append(
+            Heuristic(atom, type_, bias, priority, condition)
+        )
